VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsJsonObject"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' Autor: F. Schler (frank@activevb.de)
' Datum: 09/2022

Option Explicit

' ----==== Const ====----
Private Const WindowsDataJsonJsonObject As String = "Windows.Data.Json.JsonObject"
Private Const IID_IJsonValue As String = "{a3219ecb-f0b3-4dcd-beee-19d48cd3ed1e}"
Private Const IID_IJsonObject As String = "{064e24dd-29c2-4f83-9ac1-9ee11578beb3}"
Private Const IID_IStringable As String = "{96369f54-8eb6-48f0-abce-c1b211e627c3}"
Private Const IID_IJsonObjectStatics As String = "{2289f159-54de-45d8-abcc-22603fa066a0}"
Private Const IID_IMap_HSTRING_IJsonValue As String = "{c9d9a725-786b-5113-b4b7-9b61764c220b}"
Private Const IID_IJsonObjectWithDefaultValues As String = "{d960d2a2-b7f0-4f00-8e44-d82cf415ea13}"
Private Const IID_IIterable_IKeyValuePair_HSTRING_IJsonValue As String = "{dfabb6e1-0411-5a8f-aa87-354e7110f099}"

' ----==== Enums ====----
Private Enum vtb_Interfaces
    
    ' IJsonObjectStatics
    IJsonObjectStatics_Parse = 6
    IJsonObjectStatics_TryParse = 7
    
    ' IJsonObject
    IJsonObject_GetNamedValue = 6
    IJsonObject_SetNamedValue = 7
    IJsonObject_GetNamedObject = 8
    IJsonObject_GetNamedArray = 9
    IJsonObject_GetNamedString = 10
    IJsonObject_GetNamedNumber = 11
    IJsonObject_GetNamedBoolean = 12
    
    ' IJsonObjectWithDefaultValues
    IJsonObjectWithDefaultValues_GetNamedValueOrDefault = 6
    IJsonObjectWithDefaultValues_GetNamedObjectOrDefault = 7
    IJsonObjectWithDefaultValues_GetNamedStringOrDefault = 8
    IJsonObjectWithDefaultValues_GetNamedArrayOrDefault = 9
    IJsonObjectWithDefaultValues_GetNamedNumberOrDefault = 10
    IJsonObjectWithDefaultValues_GetNamedBooleanOrDefault = 11

    'IJsonValue
    IJsonValue_GetValueType = 6
    IJsonValue_Stringify = 7
    IJsonValue_GetString = 8
    IJsonValue_GetNumber = 9
    IJsonValue_GetBoolean = 10
    IJsonValue_GetArray = 11
    IJsonValue_GetObject = 12

    ' IStringable
    IStringable_ToString = 6
    
    ' IMap_HSTRING_IJsonValue
    IMap_HSTRING_IJsonValue_Lookup = 6
    IMap_HSTRING_IJsonValue_GetSize = 7
    IMap_HSTRING_IJsonValue_HasKey = 8
    IMap_HSTRING_IJsonValue_GetView = 9
    IMap_HSTRING_IJsonValue_Insert = 10
    IMap_HSTRING_IJsonValue_Remove = 11
    IMap_HSTRING_IJsonValue_Clear = 12
    
    ' IIterable_IKeyValuePair_HSTRING_IJsonValue
    IIterable_IKeyValuePair_HSTRING_IJsonValue_First = 6
    
    ' IIterator_IKeyValuePair_HSTRING_IJsonValue
    IIterator_IKeyValuePair_HSTRING_IJsonValue_GetCurrent = 6
    IIterator_IKeyValuePair_HSTRING_IJsonValue_GetHasCurrent = 7
    IIterator_IKeyValuePair_HSTRING_IJsonValue_MoveNext = 8
    IIterator_IKeyValuePair_HSTRING_IJsonValue_GetMany = 9

    ' IKeyValuePair_HSTRING_IJsonValue
    IKeyValuePair_HSTRING_IJsonValue_GetKey = 6
    IKeyValuePair_HSTRING_IJsonValue_GetValue = 7
    
End Enum

' ----==== Variablen ====----
Private cJsonError As clsJsonError
Private m_ErrorStatus As JsonErrorStatus
Private m_pIJsonValue As Long
Private m_pIStringable As Long
Private m_pIJsonObject As Long
Private m_pIJsonObjectStatics As Long
Private m_pIMap_HSTRING_IJsonValue As Long
Private m_pIJsonObjectWithDefaultValues As Long

' ----==== Class ====----
Private Sub Class_Initialize()
    If GetActivationFactory(WindowsDataJsonJsonObject, _
                            IID_IJsonObjectStatics, _
                            m_pIJsonObjectStatics) Then
        Dim pIJsonObject As Long
        If GetActivateInstance(WindowsDataJsonJsonObject, _
                               IID_IJsonObject, _
                               pIJsonObject) Then
            Me.IJsonObject = pIJsonObject
        End If
    End If
    Set cJsonError = New clsJsonError
End Sub

Private Sub Class_Terminate()
    Call ReleaseIfc(m_pIMap_HSTRING_IJsonValue)
    Call ReleaseIfc(m_pIJsonObjectWithDefaultValues)
    Call ReleaseIfc(m_pIStringable)
    Call ReleaseIfc(m_pIJsonValue)
    Call ReleaseIfc(m_pIJsonObject)
    Call ReleaseIfc(m_pIJsonObjectStatics)
    Set cJsonError = Nothing
End Sub

' ----==== Properties ====----
Public Property Get IJsonValue() As Long
    IJsonValue = m_pIJsonValue
End Property

Public Property Get IJsonObject() As Long
    IJsonObject = m_pIJsonObject
End Property

Public Property Let IJsonObject(ByVal pIJsonObject As Long)
    Call ReleaseIfc(m_pIMap_HSTRING_IJsonValue)
    Call ReleaseIfc(m_pIJsonObjectWithDefaultValues)
    Call ReleaseIfc(m_pIStringable)
    Call ReleaseIfc(m_pIJsonValue)
    Call ReleaseIfc(m_pIJsonObject)
    m_pIJsonObject = pIJsonObject
    If m_pIJsonObject <> 0& Then
        Call QueryIfc(m_pIJsonObject, IID_IJsonValue, m_pIJsonValue)
        Call QueryIfc(m_pIJsonObject, IID_IStringable, m_pIStringable)
        Call QueryIfc(m_pIJsonObject, IID_IJsonObjectWithDefaultValues, m_pIJsonObjectWithDefaultValues)
        Call QueryIfc(m_pIJsonObject, IID_IMap_HSTRING_IJsonValue, m_pIMap_HSTRING_IJsonValue)
    End If
End Property

Public Property Get JsonStatus() As JsonErrorStatus
    JsonStatus = m_ErrorStatus
End Property

' ----==== Functions ====----
' IJsonObjectStatics
Public Function Parse(ByVal inputStr As String) As clsJsonObject
    Dim Ret As New clsJsonObject
    If m_pIJsonObjectStatics <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(inputStr)
        If hString <> 0& Then
            Dim pIJsonObject As Long
            If Invoke(m_pIJsonObjectStatics, _
                      IJsonObjectStatics_Parse, _
                      hString, _
                      VarPtr(pIJsonObject)) = S_OK Then
                Ret.IJsonObject = pIJsonObject
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    Set Parse = Ret
End Function

Public Function TryParse(ByVal inputStr As String, _
                         ByRef result As clsJsonObject) As Boolean
    Dim Ret As Boolean
    If m_pIJsonObjectStatics <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(inputStr)
        If hString <> 0& Then
            Dim pIJsonObject As Long
            Dim succeeded As Boolean
            If Invoke(m_pIJsonObjectStatics, _
                      IJsonObjectStatics_TryParse, _
                      hString, _
                      VarPtr(pIJsonObject), _
                      VarPtr(succeeded)) = S_OK Then
                Ret = succeeded
                Set result = New clsJsonObject
                result.IJsonObject = pIJsonObject
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    TryParse = Ret
End Function

' IJsonObject
Public Function GetNamedValue(ByVal name As String) As clsJsonValue
    Dim Ret As New clsJsonValue
    If m_pIJsonObject <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(name)
        If hString <> 0& Then
            Dim pIJsonValue As Long
            If Invoke(m_pIJsonObject, _
                      IJsonObject_GetNamedValue, _
                      hString, _
                      VarPtr(pIJsonValue)) = S_OK Then
                Ret.IJsonValue = pIJsonValue
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    Set GetNamedValue = Ret
End Function

Public Function SetNamedValue(ByVal name As String, _
                              ByVal value As Object) As Boolean
    Dim Ret As Boolean
    If m_pIJsonObject <> 0& And _
       value.IJsonValue <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(name)
        If hString <> 0& Then
            If Invoke(m_pIJsonObject, _
                      IJsonObject_SetNamedValue, _
                      hString, _
                      value.IJsonValue) = S_OK Then
                Ret = True
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    SetNamedValue = Ret
End Function

Public Function GetNamedObject(ByVal name As String) As clsJsonObject
    Dim Ret As New clsJsonObject
    If m_pIJsonObject <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(name)
        If hString <> 0& Then
            Dim pIJsonObject As Long
            If Invoke(m_pIJsonObject, _
                      IJsonObject_GetNamedObject, _
                      hString, _
                      VarPtr(pIJsonObject)) = S_OK Then
                Ret.IJsonObject = pIJsonObject
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    Set GetNamedObject = Ret
End Function

Public Function GetNamedArray(ByVal name As String) As clsJsonArray
    Dim Ret As New clsJsonArray
    If m_pIJsonObject <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(name)
        If hString <> 0& Then
            Dim pIJsonArray As Long
            If Invoke(m_pIJsonObject, _
                      IJsonObject_GetNamedArray, _
                      hString, _
                      VarPtr(pIJsonArray)) = S_OK Then
                Ret.IJsonArray = pIJsonArray
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    Set GetNamedArray = Ret
End Function

Public Function GetNamedString(ByVal name As String) As String
    Dim Ret As String
    If m_pIJsonObject <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(name)
        If hString <> 0& Then
            Dim value As Long
            If Invoke(m_pIJsonObject, _
                      IJsonObject_GetNamedString, _
                      hString, _
                      VarPtr(value)) = S_OK Then
                Ret = GetWindowsString(value)
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    GetNamedString = Ret
End Function

Public Function GetNamedNumber(ByVal name As String) As Double
    Dim Ret As Double
    If m_pIJsonObject <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(name)
        If hString <> 0& Then
            Dim value As Double
            If Invoke(m_pIJsonObject, _
                      IJsonObject_GetNamedNumber, _
                      hString, _
                      VarPtr(value)) = S_OK Then
                Ret = value
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    GetNamedNumber = Ret
End Function

Public Function GetNamedBoolean(ByVal name As String) As Boolean
    Dim Ret As Boolean
    If m_pIJsonObject <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(name)
        If hString <> 0& Then
            Dim value As Boolean
            If Invoke(m_pIJsonObject, _
                      IJsonObject_GetNamedBoolean, _
                      hString, _
                      VarPtr(value)) = S_OK Then
                Ret = value
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    GetNamedBoolean = Ret
End Function

' IJsonObjectWithDefaultValues
Public Function GetNamedValueOrDefault(ByVal name As String, _
                                       ByVal defaultValue As clsJsonValue) As clsJsonValue
    Dim Ret As New clsJsonValue
    If m_pIJsonObjectWithDefaultValues <> 0& And _
       defaultValue.IJsonValue <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(name)
        If hString <> 0& Then
            Dim pIJsonValue As Long
            If Invoke(m_pIJsonObjectWithDefaultValues, _
                      IJsonObjectWithDefaultValues_GetNamedValueOrDefault, _
                      hString, _
                      defaultValue.IJsonValue, _
                      VarPtr(pIJsonValue)) = S_OK Then
                Ret.IJsonValue = pIJsonValue
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    Set GetNamedValueOrDefault = Ret
End Function

Public Function GetNamedObjectOrDefault(ByVal name As String, _
                                        ByVal defaultValue As clsJsonObject) As clsJsonObject
    Dim Ret As New clsJsonObject
    If m_pIJsonObjectWithDefaultValues <> 0& And _
       defaultValue.IJsonObject <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(name)
        If hString <> 0& Then
            Dim pIJsonObject As Long
            If Invoke(m_pIJsonObjectWithDefaultValues, _
                      IJsonObjectWithDefaultValues_GetNamedObjectOrDefault, _
                      hString, _
                      defaultValue.IJsonObject, _
                      VarPtr(pIJsonObject)) = S_OK Then
                Ret.IJsonObject = pIJsonObject
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    Set GetNamedObjectOrDefault = Ret
End Function

Public Function GetNamedStringOrDefault(ByVal name As String, _
                                        ByVal defaultValue As String) As String
    Dim Ret As String
    If m_pIJsonObjectWithDefaultValues <> 0& Then
        Dim hName As Long
        hName = CreateWindowsString(name)
        If hName <> 0& Then
            Dim hValue As Long
            hValue = CreateWindowsString(name)
            If hValue <> 0& Then
                Dim value As Long
                If Invoke(m_pIJsonObjectWithDefaultValues, _
                          IJsonObjectWithDefaultValues_GetNamedStringOrDefault, _
                          hName, _
                          hValue, _
                          VarPtr(value)) = S_OK Then
                    Ret = GetWindowsString(value)
                End If
                Call DeleteWindowsString(hValue)
            End If
            Call DeleteWindowsString(hName)
        End If
    End If
    GetNamedStringOrDefault = Ret
End Function

Public Function GetNamedArrayOrDefault(ByVal name As String, _
                                       ByVal defaultValue As clsJsonArray) As clsJsonArray
    Dim Ret As New clsJsonArray
    If m_pIJsonObjectWithDefaultValues <> 0& And _
       defaultValue.IJsonArray <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(name)
        If hString <> 0& Then
            Dim pIJsonArray As Long
            If Invoke(m_pIJsonObjectWithDefaultValues, _
                      IJsonObjectWithDefaultValues_GetNamedArrayOrDefault, _
                      hString, _
                      defaultValue.IJsonArray, _
                      VarPtr(pIJsonArray)) = S_OK Then
                Ret.IJsonArray = pIJsonArray
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    Set GetNamedArrayOrDefault = Ret
End Function

Public Function GetNamedNumberOrDefault(ByVal name As String, _
                                        ByVal defaultValue As Double) As Double
    Dim Ret As Double
    If m_pIJsonObjectWithDefaultValues <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(name)
        If hString <> 0& Then
            Dim value As Double
            If Invoke(m_pIJsonObjectWithDefaultValues, _
                      IJsonObjectWithDefaultValues_GetNamedNumberOrDefault, _
                      hString, _
                      defaultValue, _
                      VarPtr(value)) = S_OK Then
                    Ret = value
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    GetNamedNumberOrDefault = Ret
End Function

Public Function GetNamedBooleanOrDefault(ByVal name As String, _
                                         ByVal defaultValue As Boolean) As Boolean
    Dim Ret As Boolean
    If m_pIJsonObjectWithDefaultValues <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(name)
        If hString <> 0& Then
            Dim value As Boolean
            If Invoke(m_pIJsonObjectWithDefaultValues, _
                      IJsonObjectWithDefaultValues_GetNamedBooleanOrDefault, _
                      hString, _
                      defaultValue, _
                      VarPtr(value)) = S_OK Then
                Ret = value
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    GetNamedBooleanOrDefault = Ret
End Function

'IJsonValue
Public Function GetValueType() As JsonValueType
    Dim Ret As JsonValueType
    If m_pIJsonValue <> 0& Then
        Dim value As Long
        If Invoke(m_pIJsonValue, _
                  IJsonValue_GetValueType, _
                  VarPtr(value)) = S_OK Then
            Ret = value
        End If
    End If
    GetValueType = Ret
End Function

Public Function Stringify() As String
    Dim Ret As String
    If m_pIJsonValue <> 0& Then
        Dim value As Long
        If Invoke(m_pIJsonValue, _
                  IJsonValue_Stringify, _
                  VarPtr(value)) = S_OK Then
            Ret = GetWindowsString(value)
        End If
    End If
    Stringify = Ret
End Function

Public Function GetString() As String
    Dim Ret As String
    If m_pIJsonValue <> 0& Then
        Dim value As Long
        If Invoke(m_pIJsonValue, _
                  IJsonValue_GetString, _
                  VarPtr(value)) = S_OK Then
            Ret = GetWindowsString(value)
        End If
    End If
    GetString = Ret
End Function

Public Function GetNumber() As Double
    Dim Ret As Double
    If m_pIJsonValue <> 0& Then
        Dim value As Double
        If Invoke(m_pIJsonValue, _
                  IJsonValue_GetNumber, _
                  VarPtr(value)) = S_OK Then
            Ret = value
        End If
    End If
    GetNumber = Ret
End Function

Public Function GetBoolean() As Boolean
    Dim Ret As Boolean
    If m_pIJsonValue <> 0& Then
        Dim value As Boolean
        If Invoke(m_pIJsonValue, _
                  IJsonValue_GetBoolean, _
                  VarPtr(value)) = S_OK Then
            Ret = value
        End If
    End If
    GetBoolean = Ret
End Function

Public Function GetArray() As clsJsonArray
    Dim Ret As New clsJsonArray
    If m_pIJsonValue <> 0& Then
        Dim pIJsonArray As Long
        If Invoke(m_pIJsonValue, _
                  IJsonValue_GetArray, _
                  VarPtr(pIJsonArray)) = S_OK Then
            Ret.IJsonArray = pIJsonArray
        End If
    End If
    Set GetArray = Ret
End Function

Public Function GetObject() As clsJsonObject
    Dim Ret As New clsJsonObject
    If m_pIJsonValue <> 0& Then
        Dim pIJsonObject As Long
        If Invoke(m_pIJsonValue, _
                  IJsonValue_GetObject, _
                  VarPtr(pIJsonObject)) = S_OK Then
            Ret.IJsonObject = pIJsonObject
        End If
    End If
    Set GetObject = Ret
End Function

' IStringable
Public Function ToString() As String
    Dim Ret As String
    If m_pIStringable <> 0& Then
        Dim value As Long
        If Invoke(m_pIStringable, _
                  IStringable_ToString, _
                  VarPtr(value)) = S_OK Then
            Ret = GetWindowsString(value)
        End If
    End If
    ToString = Ret
End Function

' IMap_HSTRING_IJsonValue
Public Function Lookup(ByVal key As String) As clsJsonValue
    Dim Ret As New clsJsonValue
    If m_pIMap_HSTRING_IJsonValue <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(key)
        If hString <> 0& Then
            Dim pIJsonValue As Long
            If Invoke(m_pIMap_HSTRING_IJsonValue, _
                      IMap_HSTRING_IJsonValue_Lookup, _
                      hString, _
                      VarPtr(pIJsonValue)) = S_OK Then
                Ret.IJsonValue = pIJsonValue
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    Set Lookup = Ret
End Function

Public Function GetSize() As Long
    Dim Ret As Long
    If m_pIMap_HSTRING_IJsonValue <> 0& Then
        Dim value As Long
        If Invoke(m_pIMap_HSTRING_IJsonValue, _
                  IMap_HSTRING_IJsonValue_GetSize, _
                  VarPtr(value)) = S_OK Then
            Ret = value
        End If
    End If
    GetSize = Ret
End Function

Public Function HasKey(ByVal key As String) As Boolean
    Dim Ret As Boolean
    If m_pIMap_HSTRING_IJsonValue <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(key)
        If hString <> 0& Then
            Dim value As Boolean
            If Invoke(m_pIMap_HSTRING_IJsonValue, _
                      IMap_HSTRING_IJsonValue_HasKey, _
                      hString, _
                      VarPtr(value)) = S_OK Then
                Ret = value
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    HasKey = Ret
End Function

Public Function Insert(ByVal key As String, _
                       ByVal value As clsJsonValue, _
                       Optional ByRef replaced As Boolean) As Boolean
    Dim Ret As Boolean
    If m_pIMap_HSTRING_IJsonValue <> 0& And _
       value.IJsonValue <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(key)
        If hString <> 0& Then
            If Invoke(m_pIMap_HSTRING_IJsonValue, _
                      IMap_HSTRING_IJsonValue_Insert, _
                      hString, _
                      value.IJsonValue, _
                      VarPtr(replaced)) = S_OK Then
                Ret = True
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    Insert = Ret
End Function

Public Function Remove(ByVal key As String) As Boolean
    Dim Ret As Boolean
    If m_pIMap_HSTRING_IJsonValue <> 0& Then
        Dim hString As Long
        hString = CreateWindowsString(key)
        If hString <> 0& Then
            If Invoke(m_pIMap_HSTRING_IJsonValue, _
                      IMap_HSTRING_IJsonValue_Remove, _
                      hString) = S_OK Then
                Ret = True
            End If
            Call DeleteWindowsString(hString)
        End If
    End If
    Remove = Ret
End Function

Public Function Clear() As Boolean
    Dim Ret As Boolean
    If m_pIMap_HSTRING_IJsonValue <> 0& Then
        If Invoke(m_pIMap_HSTRING_IJsonValue, _
                  IMap_HSTRING_IJsonValue_Clear) = S_OK Then
            Ret = True
        End If
    End If
    Clear = Ret
End Function

' IIterable_IKeyValuePair_HSTRING_IJsonValue
' IIterator_IKeyValuePair_HSTRING_IJsonValue
' IKeyValuePair_HSTRING_IJsonValue
Friend Function GetKeyValuePair() As KeyValuePair()
    Dim Ret() As KeyValuePair
    If m_pIJsonObject <> 0& Then
        Dim count As Long
        count = Me.GetSize
        If count > 0& Then
            ReDim Ret(count - 1)
            Dim pIIterable_IKeyValuePair_HSTRING_IJsonValue As Long
            If QueryIfc(m_pIJsonObject, _
                        IID_IIterable_IKeyValuePair_HSTRING_IJsonValue, _
                        pIIterable_IKeyValuePair_HSTRING_IJsonValue) Then
                Dim pIIterator_IKeyValuePair_HSTRING_IJsonValue As Long
                If Invoke(pIIterable_IKeyValuePair_HSTRING_IJsonValue, _
                          IIterable_IKeyValuePair_HSTRING_IJsonValue_First, _
                          VarPtr(pIIterator_IKeyValuePair_HSTRING_IJsonValue)) = S_OK Then
                    Dim pIKeyValuePair_HSTRING_IJsonValue As Long
                    If Invoke(pIIterator_IKeyValuePair_HSTRING_IJsonValue, _
                              IIterator_IKeyValuePair_HSTRING_IJsonValue_GetCurrent, _
                              VarPtr(pIKeyValuePair_HSTRING_IJsonValue)) = S_OK Then
                        Dim value As Long
                        If Invoke(pIKeyValuePair_HSTRING_IJsonValue, _
                                  IKeyValuePair_HSTRING_IJsonValue_GetKey, _
                                  VarPtr(value)) = S_OK Then
                            Ret(0).key = GetWindowsString(value)
                        End If
                        If Invoke(pIKeyValuePair_HSTRING_IJsonValue, _
                                  IKeyValuePair_HSTRING_IJsonValue_GetValue, _
                                  VarPtr(value)) = S_OK Then
                            Dim JsonValue As New clsJsonValue
                            JsonValue.IJsonValue = value
                            Set Ret(0).value = JsonValue
                        End If
                        Call ReleaseIfc(pIKeyValuePair_HSTRING_IJsonValue)
                    End If
                    Dim item As Long
                    For item = 1 To count
                        Dim moveNext As Boolean
                        If Invoke(pIIterator_IKeyValuePair_HSTRING_IJsonValue, _
                                  IIterator_IKeyValuePair_HSTRING_IJsonValue_MoveNext, _
                                  VarPtr(moveNext)) = S_OK Then
                            If moveNext Then
                                Dim pIKeyValuePair_HSTRING_IJsonValue_Next As Long
                                If Invoke(pIIterator_IKeyValuePair_HSTRING_IJsonValue, _
                                          IIterator_IKeyValuePair_HSTRING_IJsonValue_GetCurrent, _
                                          VarPtr(pIKeyValuePair_HSTRING_IJsonValue_Next)) = S_OK Then
                                    Dim value_Next As Long
                                    If Invoke(pIKeyValuePair_HSTRING_IJsonValue_Next, _
                                              IKeyValuePair_HSTRING_IJsonValue_GetKey, _
                                              VarPtr(value_Next)) = S_OK Then
                                        Ret(item).key = GetWindowsString(value_Next)
                                    End If
                                    If Invoke(pIKeyValuePair_HSTRING_IJsonValue_Next, _
                                              IKeyValuePair_HSTRING_IJsonValue_GetValue, _
                                              VarPtr(value_Next)) = S_OK Then
                                        Dim JsonValue_Next As New clsJsonValue
                                        JsonValue_Next.IJsonValue = value_Next
                                        Set Ret(item).value = JsonValue_Next
                                    End If
                                    Call ReleaseIfc(pIKeyValuePair_HSTRING_IJsonValue_Next)
                                End If
                            End If
                        End If
                    Next
                    Call ReleaseIfc(pIIterator_IKeyValuePair_HSTRING_IJsonValue)
                End If
                Call ReleaseIfc(pIIterable_IKeyValuePair_HSTRING_IJsonValue)
            End If
        End If
    End If
    GetKeyValuePair = Ret
End Function

' ----==== Invoke Interface ====----
Private Function Invoke(ByVal pInterface As Long, _
                        ByVal vtb As vtb_Interfaces, _
                        ParamArray var()) As Variant
    If pInterface <> 0& Then
        Dim hResult As Long
        hResult = OleInvoke(pInterface, vtb, var)
        m_ErrorStatus = cJsonError.GetJsonStatus(hResult)
        Invoke = hResult
    End If
End Function

